/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 1997-2010 Sun Microsystems, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common Development
 * and Distribution License("CDDL") (collectively, the "License").  You
 * may not use this file except in compliance with the License. You can obtain
 * a copy of the License at https://glassfish.dev.java.net/public/CDDL+GPL.html
 * or glassfish/bootstrap/legal/LICENSE.txt.  See the License for the specific
 * language governing permissions and limitations under the License.
 *
 * When distributing the software, include this License Header Notice in each
 * file and include the License file at glassfish/bootstrap/legal/LICENSE.txt.
 * Sun designates this particular file as subject to the "Classpath" exception
 * as provided by Sun in the GPL Version 2 section of the License file that
 * accompanied this code.  If applicable, add the following below the License
 * Header, with the fields enclosed by brackets [] replaced by your own
 * identifying information: "Portions Copyrighted [year]
 * [name of copyright owner]"
 *
 * Contributor(s):
 *
 * If you wish your version of this file to be governed by only the CDDL or
 * only the GPL Version 2, indicate your decision by adding "[Contributor]
 * elects to include this software in this distribution under the [CDDL or GPL
 * Version 2] license."  If you don't indicate a single choice of license, a
 * recipient has the option to distribute your version of this file under
 * either the CDDL, the GPL Version 2 or to extend the choice of license to
 * its licensees as provided above.  However, if you add GPL Version 2 code
 * and therefore, elected the GPL Version 2 license, then the option applies
 * only if the new code is made subject to such option by the copyright
 * holder.
 */

package org.glassfish.jts.admin.cli;

import org.glassfish.api.admin.AdminCommand;
import org.glassfish.api.admin.AdminCommandContext;
import org.glassfish.api.I18n;
import org.glassfish.api.Param;
import org.glassfish.api.ActionReport;
import org.glassfish.api.admin.Cluster;
import org.glassfish.api.admin.RuntimeType;
import org.glassfish.api.admin.InstanceState;
import org.glassfish.api.admin.CommandRunner;
import org.glassfish.config.support.CommandTarget;
import org.glassfish.config.support.TargetType;
import org.jvnet.hk2.annotations.Service;
import org.jvnet.hk2.annotations.Scoped;
import org.jvnet.hk2.annotations.Inject;
import org.jvnet.hk2.component.PerLookup;
import com.sun.enterprise.transaction.api.ResourceRecoveryManager;
import com.sun.enterprise.config.serverbeans.Servers;
import com.sun.enterprise.util.i18n.StringManager;

@Service(name = "recover-transactions")
@TargetType({CommandTarget.DAS,CommandTarget.STANDALONE_INSTANCE, CommandTarget.CLUSTERED_INSTANCE})
@Cluster(RuntimeType.DAS)
@Scoped(PerLookup.class)
@I18n("recover.transactions")
public class RecoverTransactions implements AdminCommand {

    private static StringManager localStrings =
            StringManager.getManager(RecoverTransactions.class);

    @Param(name = "transactionlogdir", optional = true)
    String transactionLogDir;

    @Param(name="target", alias = "destination", optional = true)
    String destinationServer;

    @Param(name = "server_name", primary = true)
    String serverToRecover;

    @Inject
    Servers servers;

    @Inject 
    InstanceState state;

    @Inject
    CommandRunner runner;

    public void execute(AdminCommandContext context) {
        final ActionReport report = context.getActionReport();

System.err.println("==> original target: " + destinationServer + " ... server: " + serverToRecover);
        //if (domain.getServerNamed(serverToRecover) == null) {
        if (servers.getServer(serverToRecover) == null) {
            report.setMessage(localStrings.getString("recover.transactions.serverBeRecoveredIsNotKnown",
                    serverToRecover));
            report.setActionExitCode(ActionReport.ExitCode.FAILURE);
            return;
        }

        if (isServerRunning(serverToRecover)) {
            if (destinationServer != null && !serverToRecover.equals(destinationServer)) {
                report.setMessage(localStrings.getString(
                        "recover.transactions.runningServerBeRecoveredFromAnotherServer",
                        serverToRecover, destinationServer));
                report.setActionExitCode(ActionReport.ExitCode.FAILURE);
                return;
            }
            if (transactionLogDir != null) {
                report.setMessage(localStrings.getString(
                        "recover.transactions.logDirShouldNotBeSpecifiedForSelfRecovery"));
                report.setActionExitCode(ActionReport.ExitCode.FAILURE);
                return;
            }
        } else if (destinationServer == null) {
            report.setMessage(localStrings.getString("recover.transactions.noDestinationServer"));
            report.setActionExitCode(ActionReport.ExitCode.FAILURE);
            return;

        } else if (servers.getServer(destinationServer) == null) {
            report.setMessage(localStrings.getString("recover.transactions.DestinationServerIsNotKnown"));
            report.setActionExitCode(ActionReport.ExitCode.FAILURE);
            return;

        } else if (!isServerRunning(destinationServer)) {
            report.setMessage(localStrings.getString("recover.transactions.destinationServerIsNotAlive"));
            report.setActionExitCode(ActionReport.ExitCode.FAILURE);
            return;

        } else if (transactionLogDir == null) {
            report.setMessage(localStrings.getString("recover.transactions.logDirNotSpecifiedForDelegatedRecovery"));
            report.setActionExitCode(ActionReport.ExitCode.FAILURE);
            return;
        }

        //here we are only if parameters consistent
        if(destinationServer==null)
            destinationServer = serverToRecover;

        try {
            boolean result;
            final RecoverTransactionsInternal command = (RecoverTransactionsInternal) runner
                .getCommand("_recover-transactions-internal", report, context.getLogger());
            command.transactionLogDir = transactionLogDir;
            command.destinationServer = destinationServer;
            command.serverToRecover = serverToRecover;
            command.execute(context);
            if(report.getActionExitCode() != ActionReport.ExitCode.SUCCESS) {
                throw new IllegalStateException();
            }
        } catch (Exception e) {
            report.setMessage(localStrings.getString("recover.transactions.failed"));
            report.setActionExitCode(ActionReport.ExitCode.FAILURE);
            report.setFailureCause(e);
        }
    }

    // Implementation note: This has to be redone when clustering is supported in V3. Currently this implementation
    // returns true by default as it gets executed only if DAS is running.
    private boolean isServerRunning(String serverName) {
        return state.getState(serverName) == InstanceState.StateType.RUNNING;
    }

}
